/**
 * Mix reach methods
 * @function reachMix
 * @param {Object} Class
 * @returns {Object} MixedClass
 */
'use strict'

const co = require('co')
const asleep = require('asleep')

/** @lends reachMix */
function reachMix (Class) {
  return class ReachMixed extends Class {
    /**
     * Wait to reach endpoint
     * @param {Object} options
     * @returns {Promise}
     */
    reach (options = {}) {
      const s = this
      let {
        interval = 500,
        delay = 0,
        maxTry = 10
      } = options
      let { baseUrl, agent } = s
      let count = 0
      return co(function * () {
        yield asleep(delay)
        while (count < maxTry) {
          try {
            let { statusType } = yield agent.head(baseUrl)
            return statusType === 2
          } catch (e) {
            // Do Nothing
          }
          count++
          yield asleep(interval)
        }
        return false
      })
    }

  }
}

module.exports = reachMix
