/**
 * Test case for apemanApi.
 * Runs with mocha.
 */
'use strict'

const apemanApi = require('apeman-api')
const asleep = require('asleep')
const assert = require('assert')
const Client = require('../lib/apeman_api_client')
const aport = require('aport')

const co = require('co')

describe('ap-client-test', function () {
  this.timeout(3000)
  let port, baseUrl
  before(() => co(function * () {
    port = yield aport()
    baseUrl = `http://localhost:${port}`
  }))

  after(() => co(function * () {
  }))

  it('Command', () => co(function * () {
    let configuration = require.resolve('../misc/mocks/mock-Apemanfile')
    let server = yield apemanApi({
      port,
      configuration
    })
    assert(server)
    let client = new Client(`${baseUrl}/api/rpc`)
    assert.ok(client)

    // Use default scope api
    {
      let spec = yield client.describe('sign')
      assert.ok(spec)
      let sign = yield client.connect('sign')
      let result = yield sign.signin()
      assert.deepEqual(result, { success: true })
    }

    // Use default scope api with camel case names
    {
      let hogeHoge = yield client.connect('hogeHoge')
      let result = yield hogeHoge.sayHey()
      assert.deepEqual(result, 'hey')
    }

    // Use custom scope api
    {
      let admin = client.of('@admin')
      let user = yield admin.connect('user')
      let result = yield user.destroy()
      assert.ok(result)
    }

    // Use server send event
    {
      let spec = yield client.describe('sse')
      let eventSource = client.eventSource('sse')
      eventSource.addEventListener('error', (err) => {
        console.error(err)
      })
      eventSource.addEventListener('tick', (ev) => {
        let { type, data } = ev
        data = JSON.parse(data)
        assert.equal(type, 'tick')
        let { count } = data
        if (count === 1) {
          eventSource.close()
        }
        assert.ok(data)
      })
      asleep(500)
    }

    // Try to reach
    {
      let reached = yield client.reach()
      assert.ok(reached)
    }

    yield asleep(300)
    yield server.close()
  }))
})

/* global describe, before, after, it */
